import { existsAsync, mkdirAsync, readFileAsync, writeFileAsync, copyAsync } from '../src/util/fs';
import { runCommand } from '../src/common';
import { Config } from '../src/config';
import { exec } from 'child_process';
import { join, resolve } from 'path';
import { mkdirs } from 'fs-extra';
const tmp = require('tmp');

const cwd = process.cwd();

export const CORDOVA_PLUGIN_ID = 'cool-cordova-plugin';
export const APP_ID = 'com.getcapacitor.cli.test';
export const APP_NAME = 'Capacitor CLI Test';

export function makeConfig(appRoot: string): Config {
  return new Config(process.platform, appRoot, `${cwd}/bin`);
}

export async function run(appRoot: string, capCommand: string) {
  return new Promise((resolve, reject) => {
    exec(`cd "${appRoot}" && "${cwd}/bin/capacitor" ${capCommand}`, (error, stdout, stderr) => {
      if (error) {
        reject(stdout + stderr);
      } else {
        resolve(stdout);
      }
    });
  });
}

export function mktmp() {
  return new Promise((resolve, reject) => {
    tmp.dir((err, path, cleanupCallback) => {
      if (err) {
        throw err;
      }

      resolve({
        cleanupCallback,
        path
      });
    });
  });
}

const APP_INDEX = `
<!DOCTYPE html>
<html lang="en" dir="ltr">
<head>
  <meta charset="UTF-8">
  <title>Test Capacitor App</title>
</head>
<body>
  <capacitor-welcome></capacitor-welcome>
</body>
</html>
`

const APP_PACKAGE_JSON = `
{
  "name": "test-app",
  "dependencies": {
    "${CORDOVA_PLUGIN_ID}": "latest"
  }
}
`

export async function makeAppDir(monoRepoLike: boolean = false) {
  const appDirObj: any = await mktmp();
  const tmpDir = appDirObj.path;
  const rootDir = monoRepoLike ? join(tmpDir, 'test-root') : join(tmpDir, 'test-app');
  if (monoRepoLike) {
    await mkdirAsync(rootDir);
  }
  const appDir = monoRepoLike ? join(rootDir, 'test-app') : rootDir;
  await mkdirAsync(appDir);
  // Make the web dir
  await mkdirAsync(join(appDir, 'www'));
  // Make a fake index.html
  await writeFileAsync(join(appDir, 'www', 'index.html'), APP_INDEX);
  // Make a fake package.json
  await writeFileAsync(join(appDir, 'package.json'), APP_PACKAGE_JSON);

  // We use 'npm install' to install @capacitor/core and @capacitor/cli
  // Otherwise later use of 'npm install --save @capacitor/android|ios' will wipe 'node_modules/@capacitor/'
  const corePath = resolve(cwd, '../core');
  const cliPath = resolve(cwd, '../cli');
  await runCommand(`cd "${rootDir}" && npm install --save ${corePath} ${cliPath}`);

  // Make a fake cordova plugin
  const cordovaPluginPath = join(tmpDir, CORDOVA_PLUGIN_ID);
  await makeCordovaPlugin(cordovaPluginPath);

  await runCommand(`cd "${rootDir}" && npm install --save ${cordovaPluginPath}`);

  return {
    ...appDirObj,
     appDir
  };
}

const CODOVA_PLUGIN_JS = `
var exec = require('cordova/exec');
var CoolPlugin = {
    doSomethingCool: function (doOverlay) {
        exec(null, null, "CoolPlugin", "doSomethingCool", []);
    }
};
module.exports = CoolPlugin;
`;

const CORDOVA_PLUGIN_XML = `
<?xml version="1.0" encoding="UTF-8"?>

<plugin xmlns="http://apache.org/cordova/ns/plugins/1.0"
    id="${CORDOVA_PLUGIN_ID}"
    version="1.0.20">
    <name>Cool Cordova Plugin</name>
    <js-module src="plugin.js" name="coolplugin">
        <clobbers target="window.CoolPlugin" />
    </js-module>
    <platform name="android">
        <config-file target="res/xml/config.xml" parent="/*">
            <feature name="CoolPlugin">
                <param name="android-package" value="com.getcapacitor.cordova.CoolPlugin"/>
            </feature>
        </config-file>
        <source-file src="android/com/getcapacitor/CoolPlugin.java" target-dir="src/com/getcapacitor/cordova" />
    </platform>
    <platform name="ios">
         <config-file target="config.xml" parent="/*">
             <feature name="CoolPlugin">
                 <param name="ios-package" value="CoolPlugin" />
             </feature>
         </config-file>
         <source-file src="src/ios/CoolPlugin.m" />
    </platform>
</plugin>
`;

const CORDOVA_PLUGIN_PACKAGE = `
{
  "name": "${CORDOVA_PLUGIN_ID}",
  "version": "0.0.1",
  "description": "Cool Cordova plugin",
  "cordova": {
    "id": "${CORDOVA_PLUGIN_ID}",
    "platforms": [
      "android",
      "ios"
    ]
  },
  "author": "Cap tester",
  "license": "MIT"
}
`;

const CORDOVA_PLUGINS_RESOURCES_PODSPEC = `
Pod::Spec.new do |s|
  s.name = 'CordovaPluginsResources'
  s.version = '0.0.105'
  s.summary = 'Resources for Cordova plugins'
  s.social_media_url = 'http://twitter.com/getcapacitor'
  s.license = 'MIT'
  s.homepage = 'https://capacitor.ionicframework.com/'
  s.authors = { 'Ionic Team' => 'hi@ionicframework.com' }
  s.source = { :git => 'https://github.com/ionic-team/capacitor.git', :tag => s.version.to_s }
  s.resources = ['resources/*']
end
`;

const CORDOVA_PLUGINS_PODSPEC = `
Pod::Spec.new do |s|
  s.name = 'CordovaPlugins'
  s.version = '0.0.105'
  s.summary = 'Autogenerated spec'
  s.license = 'Unknown'
  s.homepage = 'https://example.com'
  s.authors = { 'Capacitor Generator' => 'hi@example.com' }
  s.source = { :git => 'https://github.com/ionic-team/does-not-exist.git', :tag => s.version.to_s }
  s.source_files = 'sources/**/*.{swift,h,m,c,cc,mm}'
  s.ios.deployment_target  = '10'
  s.dependency 'CapacitorCordova'
end`;

async function makeCordovaPlugin(cordovaPluginPath: string) {
  const iosPath = join(cordovaPluginPath, 'src', 'ios');
  const androidPath = join(cordovaPluginPath, 'android/com/getcapacitor');
  await mkdirs(cordovaPluginPath);
  await writeFileAsync(join(cordovaPluginPath, 'plugin.js'), CODOVA_PLUGIN_JS);
  await writeFileAsync(join(cordovaPluginPath, 'plugin.xml'), CORDOVA_PLUGIN_XML);
  await writeFileAsync(join(cordovaPluginPath, 'package.json'), CORDOVA_PLUGIN_PACKAGE);
  await mkdirs(iosPath);
  await mkdirs(androidPath);
  await writeFileAsync(join(iosPath, 'CoolPlugin.m'), '');
  await writeFileAsync(join(androidPath, 'CoolPlugin.java'), '');
}

class MappedFS {
  constructor(private rootDir) {
  }
  async read (path) {
    return readFileAsync(resolve(this.rootDir, path), 'utf8');
  }
  async exists(path) {
    return existsAsync(resolve(this.rootDir, path));
  }
}

export { MappedFS };
